/**
 * Copyright (C) 2008-2012 AgroSense Foundation.
 *
 * AgroSense is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * There are special exceptions to the terms and conditions of the GPLv3 as it is applied to
 * this software, see the FLOSS License Exception
 * <http://www.agrosense.eu/foss-exception.html>.
 *
 * AgroSense is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with AgroSense.  If not, see <http://www.gnu.org/licenses/>.
 */
package nl.cloudfarming.client.sensor;

import java.awt.Image;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.Icon;
import nl.cloudfarming.client.imp.AbstractImportFolderNode;
import nl.cloudfarming.client.imp.AbstractImportProcessor;
import org.netbeans.api.project.Project;
import org.openide.filesystems.FileObject;
import org.openide.loaders.DataFolder;
import org.openide.loaders.DataObject;
import org.openide.nodes.FilterNode;
import org.openide.nodes.Node;
import org.openide.util.Exceptions;
import org.openide.util.ImageUtilities;

/**
 *
 * @author Timon Veenstra <monezz@gmail.com>
 */
public abstract class AbstractSensorImportProcessor extends AbstractImportProcessor {

    public static final String SENSOR_FOLDER = "Sensor";
    private static final Logger LOGGER = Logger.getLogger(AbstractSensorImportProcessor.class.getName());
    private static final Image ICON = ImageUtilities.loadImage("nl/cloudfarming/client/sensor/sensor.png", true);
    private static Node IMPORT_NODE;

    @Override
    public void doImport(FileObject importFile, Project project) throws IllegalArgumentException {
        DataObject dataObject;
        try {
            dataObject = DataObject.find(importFile);
            copySourceFiles(dataObject, getSensorImportSubFolder(project));
        } catch (IOException ex) {
            LOGGER.log(Level.SEVERE, "failed to import sensor file {0}, exception: {1}", new Object[]{importFile.getName(), ex.getMessage()});
        }
    }

    @Override
    public Node getImportNode(Project project) {
        // FIXME: merge different import types under a single node again;
        // they were split into separate nodes/subfolders to work around issue AGROSENSE-1275.
        try {
            DataFolder dataFolder = DataFolder.findFolder(getSensorImportSubFolder(project));
            assert dataFolder != null;
            Node importNode = new ImportedSensorNode(project, dataFolder.getNodeDelegate());
            return importNode;
        } catch (IOException ex) {
            LOGGER.log(Level.SEVERE, "Could not find or create folder {0} with the exception {1}", new Object[]{SENSOR_FOLDER, ex});
            Exceptions.printStackTrace(ex);
            return null;
        }
    }

    @Override
    public Icon getIcon() {
        return ImageUtilities.image2Icon(ICON);
    }

    protected FileObject getSensorImportRootFolder(Project project) throws IOException {
        return getSubFolder(getImportRootFolder(project), SENSOR_FOLDER);
    }
    
    protected abstract FileObject getSensorImportSubFolder(Project project) throws IOException;

    private static final class ImportedSensorNode extends AbstractImportFolderNode {

        public ImportedSensorNode(Project project, Node original) {
            super(project, original, new ImportedSensorNodeChildren(original));

        }

        @Override
        public Image getIcon(int type) {
            return ICON;
        }
    }

    public static class ImportedSensorNodeChildren extends FilterNode.Children {

        public ImportedSensorNodeChildren(Node node) {
            super(node);
        }

        @Override
        protected Node[] createNodes(Node object) {
            List<Node> result = new ArrayList<>();

            for (Node node : super.createNodes(object)) {
                if (accept(node)) {
                    result.add(node);
                }
            }
            return result.toArray(new Node[0]);
        }

        private boolean accept(Node node) {
            return true;
        }
    }
}
