/**
 * Copyright (C) 2008-2012 AgroSense Foundation.
 *
 * AgroSense is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * There are special exceptions to the terms and conditions of the GPLv3 as it is applied to
 * this software, see the FLOSS License Exception
 * <http://www.agrosense.eu/foss-exception.html>.
 *
 * AgroSense is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with AgroSense.  If not, see <http://www.gnu.org/licenses/>.
 */
package nl.cloudfarming.client.sensor;

import java.awt.Dimension;
import java.util.Date;
import javax.swing.DefaultListModel;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import net.miginfocom.swing.MigLayout;
import org.jdesktop.swingx.JXDatePicker;
import org.jdesktop.swingx.JXPanel;
import org.openide.util.NbBundle;

/**
 *
 * @author Gerben Feenstra
 */
public class ExportSensorRunPanel extends JXPanel {

    // Bundle strings
    private static String JLABEL_FIELDS = NbBundle.getMessage(ExportSensorRunPanel.class, "action.exportsensorrunaverages.panelcomponent.jlabel.fields");
    private static String JLABEL_STARTDATE = NbBundle.getMessage(ExportSensorRunPanel.class, "action.exportsensorrunaverages.panelcomponent.jlabel.startdate");
    private static String JLABEL_ENDDATE = NbBundle.getMessage(ExportSensorRunPanel.class, "action.exportsensorrunaverages.panelcomponent.jlabel.enddate");
    private static String JLABEL_EXAMPLE = NbBundle.getMessage(ExportSensorRunPanel.class, "action.exportsensorrunaverages.panelcomponent.jlabel.example");
    private static int FIELD_SCROLLPANE_WIDTH = 150;
    private static int FIELD_SCROLLPANE_HEIGHT = 275;
    private static int RESULT_SCROLLPANE_WIDTH = 400;
    private static int RESULT_SCROLLPANE_HEIGHT = 150;
    // Panel + Components
    private JTable exampleTable = new JTable();
    private JList fieldList = new JList();
    private JXDatePicker startDatePicker = new JXDatePicker(new Date());
    private JXDatePicker endDatePicker = new JXDatePicker(new Date());
    private JScrollPane resultScrollPane;
    private JXPanel datesPanel;
    private JScrollPane fieldScrollPane;

    public ExportSensorRunPanel() {

        // Build panel
        this.setLayout(new MigLayout(
                "wrap 2",
                "[]20[]",
                "[][]20[]"));

        // row 1    
        this.add(new JLabel(JLABEL_FIELDS), "wrap");

        // row 2
        initFieldScrollPane();

        initDatePickerPanel();

        this.add(new JLabel(JLABEL_EXAMPLE));
        // Result table
        initExampleScrollPane();
    }

    /**
     * Create panel holding the datePickers
     * @return 
     */
    private void initDatePickerPanel() {
        if (datesPanel == null) {
            datesPanel = new JXPanel(new MigLayout("wrap 2"));
            datesPanel.add(new JLabel(JLABEL_STARTDATE));
            datesPanel.add(startDatePicker);
            datesPanel.add(new JLabel(JLABEL_ENDDATE));
            datesPanel.add(endDatePicker);
            add(this.datesPanel, "wrap");
        }
    }

    /**
     * Create ScrollPane containing all Fields in a FieldList. Retrieve model from controller
     * 
     * @return ScrollPane containing all Fields
     */
    private void initFieldScrollPane() {
        if (fieldScrollPane == null) {
            fieldScrollPane = new JScrollPane();
            fieldScrollPane.setPreferredSize(new Dimension(FIELD_SCROLLPANE_WIDTH, FIELD_SCROLLPANE_HEIGHT));
            fieldList.setModel(new DefaultListModel()); // Needs DefaultListModel!
            fieldScrollPane.setViewportView(fieldList);
            this.add(fieldScrollPane, "span 1 3"); // span 3rd row as well
            
        }
    }

    /**
     * Create ScrollPane containing the example table.
     * 
     * @return ScrollPane containing the example table
     */
    private void initExampleScrollPane() {
        if (resultScrollPane == null) {
            resultScrollPane = new JScrollPane();
            resultScrollPane.setPreferredSize(new Dimension(RESULT_SCROLLPANE_WIDTH, RESULT_SCROLLPANE_HEIGHT));
            exampleTable.setRowSelectionAllowed(false);
            exampleTable.setEnabled(false);
            resultScrollPane.setViewportView(exampleTable);
            this.add(resultScrollPane, "span 2 1");
        }
    }

    public JTable getExampleTable() {
        return exampleTable;
    }

    public JList getFieldList() {
        return fieldList;
    }

    public JXDatePicker getStartDatePicker() {
        return this.startDatePicker;
    }

    public JXDatePicker getEndDatePicker() {
        return this.endDatePicker;
    }
}
