/**
 * Copyright (C) 2008-2012 AgroSense Foundation.
 *
 * AgroSense is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * There are special exceptions to the terms and conditions of the GPLv3 as it
 * is applied to this software, see the FLOSS License Exception
 * <http://www.agrosense.eu/foss-exception.html>.
 *
 * AgroSense is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
 * A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * AgroSense. If not, see <http://www.gnu.org/licenses/>.
 */
package nl.cloudfarming.client.sensor;

import com.vividsolutions.jts.geom.Coordinate;
import eu.agrosense.client.grid.Grid;
import eu.agrosense.client.grid.GridCellSize;
import eu.agrosense.client.grid.GridProvider;
import java.awt.Point;
import java.awt.image.DataBuffer;
import java.awt.image.WritableRaster;
import java.util.List;
import nl.cloudfarming.client.model.AgroURI;
import nl.cloudfarming.client.sensor.model.SensorObservation;
import org.geotools.coverage.CoverageFactoryFinder;
import org.geotools.coverage.grid.GridCoverage2D;
import org.geotools.coverage.grid.GridCoverageFactory;
import org.geotools.geometry.jts.ReferencedEnvelope;
import org.geotools.referencing.crs.DefaultGeographicCRS;
import org.opengis.geometry.BoundingBox;
import org.openide.util.Lookup;

/**
 *
 * @author Timon Veenstra
 */
public class RasterFactory {

    private RasterFactory() {
    }

    /**
     * create a raster from sensor data
     *
     *
     * @param sensorData
     * @return raster
     */
    public static GridCoverage2D createRaster(List<SensorObservation> sensorData) {
        //FIXME: gives NaN on list of size one. For now if size is 1 return null.
        if (sensorData.isEmpty() || sensorData.size() == 1) {
            return null;
        }

        double minx = sensorData.get(0).getLongitude();
        double maxx = sensorData.get(0).getLongitude();
        double miny = sensorData.get(0).getLatitude();
        double maxy = sensorData.get(0).getLatitude();

        for (SensorObservation sd : sensorData) {
            minx = (minx < sd.getLongitude()) ? minx : sd.getLongitude();
            miny = (miny < sd.getLatitude()) ? miny : sd.getLatitude();
            maxx = (maxx > sd.getLongitude()) ? maxx : sd.getLongitude();
            maxy = (maxy > sd.getLatitude()) ? maxy : sd.getLatitude();
        }

//        // width height and ration in coordiantes
//        double diffx = maxx - minx;
//        double diffy = maxy - miny;
//        double ratio = diffy / diffx;


//        // in pixels
//        Double width = Math.ceil(Math.sqrt(sensorData.size() / ratio));
//        Double height = Math.ceil(Math.sqrt(sensorData.size() / ratio));
//
//        int w = width.intValue();
//        int h = height.intValue();


        GridProvider gridProvider = Lookup.getDefault().lookup(GridProvider.class);
        //FIXME get eip from project
        Grid grid = gridProvider.getDefaultGridSection(new AgroURI("OTHER/0/FRM/1"), new Coordinate(minx, maxy), new Coordinate(maxx, miny));

//        int bands = 1;
//        Point location = new Point(0, 0);
//        WritableRaster raster = WritableRaster.createBandedRaster(DataBuffer.TYPE_INT, w, h, bands, location);

        for (SensorObservation sd : sensorData) {
//            Double x = Math.floor((sd.getLongitude() - minx) / (diffx / (width - 1)));
//            Double y = Math.floor((sd.getLatitude() - miny) / (diffy / (height - 1)));
//
//            //TODO why is y-axis flipped????? flipback here shouldnt be needed
//            y = Math.abs(y + 1 - height);

            //TODO sample data gets casted into an int, so multiply with 1000 for example
            if (sd.getNdvi() != null) {
                grid.setRasterValue(new Coordinate(sd.getLongitude(),sd.getLatitude()),(int)(sd.getNdvi() * SensorPalette.SENSOR_DATA_MULTIPLIER));
//                raster.setSample(x.intValue(), y.intValue(), 0, sd.getNdvi() * SensorPalette.SENSOR_DATA_MULTIPLIER);
            }

            if (sd.getWdvi() != null) {
                grid.setRasterValue(new Coordinate(sd.getLongitude(),sd.getLatitude()),(int)(sd.getWdvi() * SensorPalette.SENSOR_DATA_MULTIPLIER));
//                raster.setSample(x.intValue(), y.intValue(), 0, sd.getWdvi() * SensorPalette.SENSOR_DATA_MULTIPLIER);
            }

//            raster.setSample(x.intValue(), y.intValue(), 2, sd.getReading1());
//            raster.setSample(x.intValue(), y.intValue(), 3, sd.getReading2());
//            raster.setSample(x.intValue(), y.intValue(), 4, sd.getReading3());           
        }
//
//        BoundingBox boundingBox = new ReferencedEnvelope(minx, maxx, miny, maxy, DefaultGeographicCRS.WGS84);
//
//
//        GridCoverageFactory factory = CoverageFactoryFinder.getGridCoverageFactory(null);
//        return factory.create("name", raster, boundingBox);
        return grid.getGrid();
    }
}
