/**
 * Copyright (C) 2008-2012 AgroSense Foundation.
 *
 * AgroSense is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * There are special exceptions to the terms and conditions of the GPLv3 as it
 * is applied to this software, see the FLOSS License Exception
 * <http://www.agrosense.eu/foss-exception.html>.
 *
 * AgroSense is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
 * A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * AgroSense. If not, see <http://www.gnu.org/licenses/>.
 */
package nl.cloudfarming.client.sensor;

import eu.agrosense.client.grid.Grid;
import java.awt.BorderLayout;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.logging.Logger;
import javax.swing.*;
import org.netbeans.core.spi.multiview.CloseOperationState;
import org.netbeans.core.spi.multiview.MultiViewElement;
import org.netbeans.core.spi.multiview.MultiViewElementCallback;
import org.openide.awt.UndoRedo;
import org.openide.util.Lookup;
import org.openide.util.NbBundle;
import org.openide.windows.TopComponent;

/**
 * <p>Visual element for sensor data.</p> <p>Expects {@link SensorDataObject} in
 * the lookup</p> <p>Add following to your class to register multiviewelement
 * @MultiViewElement.Registration(displayName = "your display name", iconBase =
 * your icon base, mimeType = your mime type, persistenceType =
 * TopComponent.PERSISTENCE_NEVER, preferredID = "your id", position = 100) </p>
 *
 * @author Merijn Zengers <Merijn.Zengers@gmail.nl>
 */
@NbBundle.Messages("SensorDataVisualElement no preview message=No preview available")
public abstract class SensorDataVisualElement implements MultiViewElement {

    private static final Logger LOGGER = Logger.getLogger("nl.cloudfarming.client.sensor");
    private SensorDataObject obj;
    private JToolBar toolbar = new JToolBar();
    private transient MultiViewElementCallback callback;
    private JPanel visualPanel;

    /**
     * <p>Expects {@link SensorDataObject} in the passed lookup</p>
     */
    public SensorDataVisualElement(Lookup lkp) {
        obj = lkp.lookup(SensorDataObject.class);
        assert obj != null;

    }

    public void updateName() {
        if (callback != null) {
            String name = getLookup().lookup(SensorDataObject.class).getPrimaryFile().getNameExt();
            TopComponent tc = callback.getTopComponent();
            tc.setHtmlDisplayName(name);
            tc.setDisplayName(name);
            tc.setName(getName());
            tc.setToolTipText(getName());
        }
    }

    public abstract String getName();

    @Override
    public JComponent getVisualRepresentation() {
        if (visualPanel == null) {
            visualPanel = new JPanel(new BorderLayout());
            Grid grid = obj.getSensorGrid();
            if (grid != null) {
                final GridCoveragePanel gridCoveragePanel = new GridCoveragePanel(grid, obj.getPalette());
                obj.addPropertyChangeListener(new PropertyChangeListener() {

                    @Override
                    public void propertyChange(PropertyChangeEvent pce) {
                        if (SensorDataObject.PROP_PALETTE_CHANGED.equals(pce.getPropertyName())) {
                            gridCoveragePanel.setPalette(obj.getPalette());
                            gridCoveragePanel.repaint();
                            visualPanel.revalidate();
                        }
                    }
                });
                
                visualPanel.add(gridCoveragePanel);
            } else {
                visualPanel.add(new JLabel(Bundle.SensorDataVisualElement_no_preview_message()), BorderLayout.NORTH);
            }
        }
        return visualPanel;
    }

    @Override
    public JComponent getToolbarRepresentation() {
        return toolbar;
    }

    @Override
    public Action[] getActions() {
        return new Action[0];
    }

    @Override
    public Lookup getLookup() {
        return obj.getLookup();
    }

    @Override
    public void componentOpened() {
    }

    @Override
    public void componentClosed() {
    }

    @Override
    public void componentShowing() {
        if (callback != null) {
            updateName();
        }
    }

    @Override
    public void componentHidden() {
    }

    @Override
    public void componentActivated() {
    }

    @Override
    public void componentDeactivated() {
    }

    @Override
    public UndoRedo getUndoRedo() {
        return UndoRedo.NONE;
    }

    @Override
    public void setMultiViewCallback(MultiViewElementCallback callback) {
        this.callback = callback;
    }

    @Override
    public CloseOperationState canCloseElement() {
        return CloseOperationState.STATE_OK;
    }
}
