/**
 * Copyright (C) 2008-2012 AgroSense Foundation.
 *
 * AgroSense is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * There are special exceptions to the terms and conditions of the GPLv3 as it
 * is applied to this software, see the FLOSS License Exception
 * <http://www.agrosense.eu/foss-exception.html>.
 *
 * AgroSense is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
 * A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * AgroSense. If not, see <http://www.gnu.org/licenses/>.
 */
package nl.cloudfarming.client.sensor.visual;

import java.util.List;
import javax.swing.*;
import javax.swing.table.DefaultTableModel;
import net.miginfocom.swing.MigLayout;
import nl.cloudfarming.client.sensor.model.Measurement;
import nl.cloudfarming.client.sensor.model.MeasurementCollection;
import nl.cloudfarming.client.util.swing.BodyPanel;
import org.netbeans.core.spi.multiview.CloseOperationState;
import org.netbeans.core.spi.multiview.MultiViewElement;
import org.netbeans.core.spi.multiview.MultiViewElementCallback;
import org.openide.awt.UndoRedo;
import org.openide.util.ImageUtilities;
import org.openide.util.Lookup;
import org.openide.util.NbBundle.Messages;
import org.openide.windows.TopComponent;

/**
 * Visual element to display measurements
 *
 * @author Merijn Zengers <Merijn.Zengers@gmail.nl>
 */
@MultiViewElement.Registration(displayName = "#MeasurementsDataElement_display_name",
iconBase = MeasurementsDataElement.ICON_PATH,
mimeType = Measurement.MIME_TYPE,
persistenceType = TopComponent.PERSISTENCE_NEVER,
preferredID = "nl.cloudfarming.client.sensor.visual.MeasurementsDataElement",
position = 100)
@Messages("MeasurementsDataElement_display_name=Measurement details")
public class MeasurementsDataElement implements MultiViewElement {

    private JToolBar toolbar = new JToolBar();
    private Lookup lkp;
    private transient MultiViewElementCallback callback;
    private MeasurementCollection collection;
    private BodyPanel visualPanel;
    public static final String ICON_PATH = "nl/cloudfarming/client/sensor/visual/measurement.png";
    public static final String ICON_PATH_32 = "nl/cloudfarming/client/sensor/visual/measurement32.png";
    private static final ImageIcon ICON = ImageUtilities.loadImageIcon(ICON_PATH_32, true);

    public MeasurementsDataElement(Lookup lkp) {
        this.lkp = lkp;
        collection = lkp.lookup(MeasurementCollection.class);
        assert collection != null : "Cannot create measurement view without MeasurementCollection in the lookup";
    }

    @Override
    public JComponent getVisualRepresentation() {
        if (visualPanel == null) {
            visualPanel = BodyPanel.create(new MeasurementsVisualPanel(collection.getMeasurements()));
            visualPanel.setIcon(ICON);
            visualPanel.setTitle(Bundle.MeasurementsDataElement_display_name());
        }
        return visualPanel;
    }

    @Override
    public JComponent getToolbarRepresentation() {
        return toolbar;
    }

    @Override
    public Action[] getActions() {
        return new Action[0];
    }

    @Override
    public Lookup getLookup() {
        return lkp;
    }

    @Override
    public void componentOpened() {
    }

    @Override
    public void componentClosed() {
    }

    @Override
    public void componentShowing() {
        updateName();
    }

    public void updateName() {
        if (callback != null) {
            String name = collection.getMeasurementType().toString();
            TopComponent tc = callback.getTopComponent();
            tc.setHtmlDisplayName(name);
            tc.setDisplayName(name);
            tc.setName(name);
            tc.setToolTipText(name);
        }
    }

    @Override
    public void componentHidden() {
    }

    @Override
    public void componentActivated() {
    }

    @Override
    public void componentDeactivated() {
    }

    @Override
    public UndoRedo getUndoRedo() {
        return UndoRedo.NONE;
    }

    @Override
    public void setMultiViewCallback(MultiViewElementCallback callback) {
        this.callback = callback;
    }

    @Override
    public CloseOperationState canCloseElement() {
        return CloseOperationState.STATE_OK;
    }

    private class MeasurementsVisualPanel extends JPanel {

        private final String[] COLUMN_NAMES = new String[]{"Value"};

        public MeasurementsVisualPanel(List<Measurement> measurements) {
            setLayout(new MigLayout("wrap 1"));
            DefaultTableModel measurementsTableModel = new DefaultTableModel(COLUMN_NAMES, 0);
            for (Measurement measurement : measurements) {
                measurementsTableModel.addRow(new Double[]{measurement.getValue()});
            }
            JTable measurementsTable = new JTable(measurementsTableModel);
            measurementsTable.setEnabled(false);
            JScrollPane scrollPane = new JScrollPane(measurementsTable);
            this.add(scrollPane);
        }
    }
}
