/**
 * Copyright (C) 2008-2012 AgroSense Foundation.
 *
 * AgroSense is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * There are special exceptions to the terms and conditions of the GPLv3 as it
 * is applied to this software, see the FLOSS License Exception
 * <http://www.agrosense.eu/foss-exception.html>.
 *
 * AgroSense is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
 * A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * AgroSense. If not, see <http://www.gnu.org/licenses/>.
 */
package nl.cloudfarming.client.sensor.visual;

import com.vividsolutions.jts.geom.Geometry;
import eu.agrosense.client.grid.Grid;
import java.awt.Component;
import java.text.MessageFormat;
import java.util.ArrayList;
import java.util.List;
import javax.swing.JComponent;
import nl.cloudfarming.client.model.AgroURI;
import nl.cloudfarming.client.sensor.model.MeasurementCollection;
import nl.cloudfarming.client.sensor.model.Observation;
import nl.cloudfarming.client.sensor.model.ObservationManager;
import nl.cloudfarming.client.sensor.visual.util.ObservationUtil;
import org.openide.DialogDisplayer;
import org.openide.WizardDescriptor;
import org.openide.util.HelpCtx;
import org.openide.util.Lookup;
import org.openide.util.actions.NodeAction;

/**
 * Create a new observation
 *
 * @author Merijn Zengers <Merijn.Zengers@gmail.nl>
 */
public abstract class NewObservationAction extends NodeAction {

    private final String icon;
    private final String title;

    public NewObservationAction(String icon, String title) {
        putValue(NAME, title);
        this.icon = icon;
        this.title = title;
    } 

    public void performAction(Geometry geometry, AgroURI sourceUri) {
        assert geometry != null : "Cannot perform action without geometry";
        List<WizardDescriptor.Panel<WizardDescriptor>> panels = new ArrayList<>();
        Observation observation = new Observation(sourceUri, sourceUri);
        observation.setSourceGeometry(geometry);
        panels.add(new NewObservationWizardPanel(icon, title, observation));
        String[] steps = new String[panels.size()];
        for (int i = 0; i < panels.size(); i++) {
            Component c = panels.get(i).getComponent();
            // Default step name to component name of panel.
            steps[i] = c.getName();
            if (c instanceof JComponent) { // assume Swing components
                JComponent jc = (JComponent) c;
                jc.putClientProperty(WizardDescriptor.PROP_CONTENT_SELECTED_INDEX, i);
                jc.putClientProperty(WizardDescriptor.PROP_CONTENT_DATA, steps);
                jc.putClientProperty(WizardDescriptor.PROP_AUTO_WIZARD_STYLE, true);
                jc.putClientProperty(WizardDescriptor.PROP_CONTENT_DISPLAYED, true);
                jc.putClientProperty(WizardDescriptor.PROP_CONTENT_NUMBERED, true);
            }
        }
        WizardDescriptor wiz = new WizardDescriptor(new WizardDescriptor.ArrayIterator<>(panels));
        // {0} will be replaced by WizardDesriptor.Panel.getComponent().getName()
        wiz.setTitleFormat(new MessageFormat("{0}"));
        wiz.setTitle(title);
        if (DialogDisplayer.getDefault().notify(wiz) == WizardDescriptor.FINISH_OPTION) {
            ObservationManager manager = Lookup.getDefault().lookup(ObservationManager.class);
            assert manager != null : "Observation manager must be found";
            
            for (MeasurementCollection collection : observation.getMeasurementCollections()) {
                Grid grid = ObservationUtil.createGrid(observation, collection.getMeasurements());
                collection.setGrid(grid);
            }

            manager.save(observation);
        }
    }

    @Override
    public String getName() {
        return title;
    }

    @Override
    public HelpCtx getHelpCtx() {
        return HelpCtx.DEFAULT_HELP;
    }
}
