/**
 * Copyright (C) 2008-2012 AgroSense Foundation.
 *
 * AgroSense is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * There are special exceptions to the terms and conditions of the GPLv3 as it
 * is applied to this software, see the FLOSS License Exception
 * <http://www.agrosense.eu/foss-exception.html>.
 *
 * AgroSense is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
 * A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * AgroSense. If not, see <http://www.gnu.org/licenses/>.
 */
package nl.cloudfarming.client.sensor.visual;

import java.awt.Component;
import java.util.ArrayList;
import java.util.List;
import javax.swing.event.ChangeListener;
import nl.cloudfarming.client.sensor.model.Measurement;
import nl.cloudfarming.client.sensor.model.Observation;
import nl.cloudfarming.client.util.swing.BodyPanel;
import org.openide.WizardDescriptor;
import org.openide.WizardValidationException;
import org.openide.util.HelpCtx;
import org.openide.util.ImageUtilities;
import org.openide.util.NbBundle;

/**
 *
 * @author Merijn Zengers <Merijn.Zengers@gmail.nl>
 */

@NbBundle.Messages({"NewObservationWizardPanel step name=Enter measurements"})
public class NewObservationWizardPanel extends ObservationVisualController implements WizardDescriptor.ValidatingPanel<WizardDescriptor> {
    
    private List<ChangeListener> listeners = new ArrayList<>();
    private BodyPanel component;
    private final String ICON;
    private final String TITLE;

    public NewObservationWizardPanel(String icon, String title, Observation observation) {
        super(observation);
        this.ICON = icon;
        this.TITLE = title;
    }
    
    @Override
    public Component getComponent() {
        if (component == null) {
            component = BodyPanel.create(getPanel());
            component.setTitle(TITLE);
            component.setIcon(ImageUtilities.loadImageIcon(ICON, false));
            component.setName(Bundle.NewObservationWizardPanel_step_name());
        }
        return component;
    }

    @Override
    public HelpCtx getHelp() {
        return HelpCtx.DEFAULT_HELP;
    }

    @Override
    public void readSettings(WizardDescriptor settings) {
    }

    @Override
    public void storeSettings(WizardDescriptor settings) {
    }

    @Override
    public boolean isValid() {
        return true;
    }

    @Override
    public void addChangeListener(ChangeListener l) {
        listeners.add(l);
    }

    @Override
    public void removeChangeListener(ChangeListener l) {
        listeners.remove(l);
    }

    private void updateListeners() {
        for (ChangeListener listener : listeners) {
            listener.stateChanged(null);
        }
    }

    @Override
    public void validate() throws WizardValidationException {
        validatePanel();
        if (!isPanelValid()) {
            throw new WizardValidationException(getPanel(), "", "");
        }
    }

    @Override
    public void panelChanged() {
        updateListeners();
    }
}
