/**
 * Copyright (C) 2008-2012 AgroSense Foundation.
 *
 * AgroSense is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * There are special exceptions to the terms and conditions of the GPLv3 as it
 * is applied to this software, see the FLOSS License Exception
 * <http://www.agrosense.eu/foss-exception.html>.
 *
 * AgroSense is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
 * A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * AgroSense. If not, see <http://www.gnu.org/licenses/>.
 */
package nl.cloudfarming.client.sensor.visual;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import javax.swing.Action;
import javax.swing.ImageIcon;
import javax.swing.JComponent;
import javax.swing.JToolBar;
import nl.cloudfarming.client.sensor.ObservationDataObject;
import nl.cloudfarming.client.sensor.model.Observation;
import nl.cloudfarming.client.util.swing.BodyPanel;
import org.netbeans.core.spi.multiview.CloseOperationState;
import org.netbeans.core.spi.multiview.MultiViewElement;
import org.netbeans.core.spi.multiview.MultiViewElementCallback;
import org.openide.awt.UndoRedo;
import org.openide.util.ImageUtilities;
import org.openide.util.Lookup;
import org.openide.util.NbBundle;
import org.openide.windows.TopComponent;

/**
 *
 * @author Merijn Zengers <Merijn.Zengers@gmail.nl>
 */
@MultiViewElement.Registration(displayName = "#ObservationDataElement_display_name",
iconBase = ObservationDataElement.ICON_PATH,
mimeType = ObservationDataObject.MIME_TYPE,
persistenceType = TopComponent.PERSISTENCE_NEVER,
preferredID = "nl.cloudfarming.client.sensor.visual.ObservationDataElement",
position = 100)
@NbBundle.Messages("ObservationDataElement_display_name=Observation details")
public class ObservationDataElement implements MultiViewElement {

    public static final String ICON_PATH = "nl/cloudfarming/client/sensor/visual/observation.png";
    public static final String ICON_PATH_32 = "nl/cloudfarming/client/sensor/visual/observation32.png";
    private static final ImageIcon ICON = ImageUtilities.loadImageIcon(ICON_PATH_32, true);
    private JToolBar toolbar = new JToolBar();
    private Lookup lkp;
    private transient MultiViewElementCallback callback;
    private Observation observation;
    private BodyPanel visualPanel;

    public ObservationDataElement(Lookup lkp) {
        this.lkp = lkp;
        this.observation = lkp.lookup(Observation.class);
        assert observation != null : "Cannot instantiate ObservationDataElement without an observation in the lookup";
        observation.addPropertyChangeListener(new ObservationNameListener());
    }

    @Override
    public JComponent getVisualRepresentation() {
        if (visualPanel == null) {
            ObservationVisual controller = new ObservationVisual(observation);
            visualPanel = BodyPanel.create(controller.getPanel());
            visualPanel.setIcon(ICON);
            visualPanel.setTitle(Bundle.ObservationDataElement_display_name());
        }
        return visualPanel;
    }

    @Override
    public JComponent getToolbarRepresentation() {
        return toolbar;
    }

    @Override
    public Action[] getActions() {
        return new Action[0];
    }

    @Override
    public Lookup getLookup() {
        return lkp;
    }

    @Override
    public void componentOpened() {
    }

    @Override
    public void componentClosed() {
    }

    @Override
    public void componentShowing() {
        updateName();
    }

    @Override
    public void componentHidden() {
    }

    @Override
    public void componentActivated() {
    }

    @Override
    public void componentDeactivated() {
    }

    @Override
    public UndoRedo getUndoRedo() {
        return UndoRedo.NONE;
    }

    @Override
    public void setMultiViewCallback(MultiViewElementCallback callback) {
        this.callback = callback;
    }

    @Override
    public CloseOperationState canCloseElement() {
        return CloseOperationState.STATE_OK;
    }

    public void updateName() {
        if (callback != null) {
            String name = observation.getName();
            TopComponent tc = callback.getTopComponent();
            tc.setHtmlDisplayName(name);
            tc.setDisplayName(name);
            tc.setName(name);
            tc.setToolTipText(name);
        }
    }

    private class ObservationNameListener implements PropertyChangeListener{

        @Override
        public void propertyChange(PropertyChangeEvent evt) {
            updateName();
        }
        
    }
}
